<?php

namespace FiloBlu\Refilo\Model\Exporter;

use Blackbird\ContentManager\Model\ResourceModel\Content\CollectionFactory;
use Blackbird\ContentManager\Model\ResourceModel\ContentType\CollectionFactory as ContentTypeCollectionFactory;
use FiloBlu\Core\Model\Configuration;
use FiloBlu\Refilo\Helper\ContentManager\Data as ContentManagerHelper;
use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\StoreManagerInterface;
use MongoDB\Driver\Exception\Exception;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class ContentManager
 * @package FiloBlu\Refilo\Model\Exporter
 */
class ContentManager extends AbstractCmsExporter
{
    /**
     * @var CollectionFactory
     */
    protected $contentCollectionFactory;
    /**
     * @var
     */
    protected $contentTypes;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var ContentTypeCollectionFactory
     */
    private $contentTypeCollectionFactory;
    /**
     * @var ContentManagerHelper
     */
    private $contentManagerHelper;
    /**
     * @var \Blackbird\ContentManager\Model\ResourceModel\ContentType\CustomField\CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var array | null
     */
    private $fieldTypeLookup;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param CollectionFactory $contentCollectionFactory
     * @param ContentTypeCollectionFactory $contentTypeCollectionFactory
     * @param ContentManagerHelper $contentManagerHelper
     * @param Configuration $coreConfiguration
     * @param \Blackbird\ContentManager\Model\ResourceModel\ContentType\CustomField\CollectionFactory $collectionFactory
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        CollectionFactory $contentCollectionFactory,
        ContentTypeCollectionFactory $contentTypeCollectionFactory,
        ContentManagerHelper $contentManagerHelper,
        Configuration $coreConfiguration,
        \Blackbird\ContentManager\Model\ResourceModel\ContentType\CustomField\CollectionFactory $collectionFactory

    )
    {
        parent::__construct($mongo, $coreConfiguration, $scopeConfig, $storeManager);
        $this->resourceConnection = $resourceConnection;
        $this->contentCollectionFactory = $contentCollectionFactory;
        $this->contentTypeCollectionFactory = $contentTypeCollectionFactory;
        $this->contentManagerHelper = $contentManagerHelper;
        $this->collectionFactory = $collectionFactory;
    }

    public function getMetaCode(): string
    {
        return 'master_data';
    }

    public function getCollectionPrefix(): string
    {
        return 'master_data_';
    }

    /**
     * @throws LocalizedException
     */
    public function execute(OutputInterface $output = null)
    {
        $this->fieldTypeLookup = [];

        foreach($this->collectionFactory->create() as $customField) {
            $this->fieldTypeLookup[$customField->getType()][] = $customField->getIdentifier();
        }

        foreach ($this->getStoreManager()->getStores() as $store) {

            $bulkData = [];
            $result = $this->contentCollectionFactory->create()->addStoreFilter($store)->addAttributeToSelect('*');
            foreach ($result as $item) {

                $itemIdentifier = $this->contentManagerHelper->getIdentifierFromId($item->getData('ct_id'));

                $bulkData[] = [
                    '_id'             => $itemIdentifier . '_' . $item->getId() . '_' . $store->getData('code'),
                    'metacode'        => 'master_data',
                    'contentmetacode' => $itemIdentifier,
                    'idx'             => $item->getId(),
                    'metadata'        => $this->getContentMetadata($item->getData('ct_id'), $item)
                ];
            }

            try {
                $this->getConnection()->update(
                    '_id',
                    $bulkData,
                    'master_data_' . $store->getCode(),
                    true,
                    $this->getDatabaseByWebsiteCode($store->getWebsite()->getCode())

                );
            } catch (Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $ctId
     * @param $item
     * @return array
     */
    public function getContentMetadata($ctId, $item)
    {
        $data = array_merge($this->contentManagerHelper->getDataFromId($ctId), $item->getData());

        if(!isset( $data['meta_description']) || empty( $data['meta_description'])) {
            $data['meta_description'] = $data['description'];
        }

        // Ensure types

        if(isset($this->fieldTypeLookup['integer'] )) {

            foreach ($this->fieldTypeLookup['integer'] as $field) {
                if(!isset($data[$field])) {
                    continue;
                }

                $data[$field] = (int)$data[$field];
            }
        }

        return $data;
    }
}
